/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2004 Dr. Lars H. Hahn

*/

package haushalt.auswertung;

import haushalt.auswertung.bloecke.LeererBlock;
import haushalt.auswertung.bloecke.TabellenBlock;
import haushalt.auswertung.bloecke.TextBlock;
import haushalt.daten.Datenbasis;
import haushalt.daten.Euro;
import haushalt.daten.Kategorie;
import haushalt.daten.zeitraum.AbstractZeitraum;
import haushalt.daten.zeitraum.Jahr;
import haushalt.gui.Haushalt;
import haushalt.gui.generischerdialog.AbstractGDPane;
import haushalt.gui.generischerdialog.EinOderAlleRegisterGDP;
import haushalt.gui.generischerdialog.MehrereKategorienGDP;
import haushalt.gui.generischerdialog.ZeitraumGDP;
import haushalt.gui.generischerdialog.ZahlGDP;

import java.awt.Font;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */
public class BAKategorieSummen extends AbstractBlockAuswertung {
  private final boolean DEBUG = false;

	public static final String ueberschrift = "Tabelle Kategorie-Summen";
	
  public BAKategorieSummen(Haushalt haushalt, Datenbasis db, String name) {
    super(haushalt, db, name);
    AbstractGDPane[] panes = new AbstractGDPane[4];
		panes[0] = new ZeitraumGDP("Erster Zeitraum:", new Jahr(2004));
		panes[1] = new ZahlGDP("Anzahl Zeitrume:", new Integer(4));
		panes[2] = new EinOderAlleRegisterGDP("Register:", db.getRegisterNamen(), null);
    panes[3] = new MehrereKategorienGDP("Kategorien:", db.getKategorien());
		erzeugeEigenschaften(haushalt.getFrame(), ueberschrift, panes);
  }

  protected String berechneAuswertung(Object[] werte) {
  	AbstractZeitraum zeitraum = (AbstractZeitraum) werte[0];
  	int anzahlZeitraeume = ((Integer) werte[1]).intValue();
    String register = (String) werte[2];
    Kategorie[] posKategorien = (Kategorie[]) werte[3];

    AbstractZeitraum tmpZeitraum = zeitraum;
		int anzahlKategorien = posKategorien.length;
		tabelle = new String[anzahlKategorien+2][anzahlZeitraeume+1];
		tabelle[0][0] = "Kategorie";
		tabelle[anzahlKategorien+1][0] = "S U M M E";

		for(int i=0;i<anzahlKategorien;i++)
			tabelle[i+1][0] = ""+posKategorien[i];
			
		for(int i=0; i<anzahlZeitraeume; i++) {
			tabelle[0][i+1] = ""+tmpZeitraum;
			Euro[] katSummen = db.getKategorieSalden(posKategorien, tmpZeitraum, register);
			Euro summe = new Euro();
			for(int j=0;j<anzahlKategorien;j++) {
				tabelle[j+1][i+1] = ""+katSummen[j];
				summe.sum(katSummen[j]);
			}
			tabelle[anzahlKategorien+1][i+1] = ""+summe;
			tmpZeitraum = tmpZeitraum.folgeZeitraum();
		}
		
		if(DEBUG) {
		  System.out.println("Kategorie-Summen berechnet.");
			System.out.println(""+anzahlKategorien+" Kategorien.");
			System.out.println(""+anzahlZeitraeume+" Zeitrume.");
		}
				
		// Vorhandene Blcke lschen und neu berechnete einfgen
		String titel = "Kategorien-Summen ("+tabelle[0][1]+" bis "+tabelle[0][anzahlZeitraeume];
		if(register == null)
		  titel += ")";
		else
		  titel += ", "+register+")";
		loescheBloecke();
		TextBlock block1 = new TextBlock(titel);
		block1.setFont(new Font(haushalt.getFontname(), Font.BOLD, haushalt.getFontgroesse()+6));
		addDokumentenBlock(block1);
		addDokumentenBlock(new LeererBlock(1));
		TabellenBlock block2 = new TabellenBlock(tabelle);
		block2.setFont(new Font(haushalt.getFontname(), Font.PLAIN, haushalt.getFontgroesse()));
    final int[] attribute = new int[anzahlZeitraeume+1];
    for(int i=0; i<=anzahlZeitraeume; i++)
      if(i == 0)
      	attribute[i] = TabellenBlock.LINKS;
      else
        attribute[i] = TabellenBlock.RECHTS;
    block2.setAusrichtung(attribute);
		addDokumentenBlock(block2);
    return titel;
  }

}
