/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2004 Dr. Lars H. Hahn

*/

package haushalt.daten;

/**
 * Verwaltet alle Kategorien.
 * Standardmig steht die die Kategorie "Sonstiges" zur Verfgung.
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */

public class Kategorie implements Comparable {

	public final static Kategorie SONSTIGES = new Kategorie("Sonstiges", null);
  
  private final Kategorie hauptkategorie;
	private String name;

  public Kategorie(String name, Kategorie hauptkategorie) {
    this.name = name;
    this.hauptkategorie = hauptkategorie;
  }

  public String getName() {
    return name;
  }
  
  public void setName(String name) {
    this.name = name;
  }
  
  public String toString() {
    if(hauptkategorie != null)
      return ""+hauptkategorie+":"+name;
    return name;
  }

	public boolean isHauptkategorie() {
		return hauptkategorie == null;
	}

  public Kategorie getHauptkategorie() {
    if(isHauptkategorie())
      return this;
    return hauptkategorie.getHauptkategorie();
  }
  
  public boolean istInKategorie(Kategorie kategorie, boolean unterkategorienVerwenden) {
  	if(kategorie == this)
  	  return true;
  	// wenn die Unterkatgorien NICHT verwendet werden, dann
  	// muss berprft werden, ob die Hauptkategorie passt:
  	if((!unterkategorienVerwenden) && (getHauptkategorie() == kategorie))
  	  return true;
  	return false;
  }
  
  // -- Kategorie-Summe -------------------------------------------------------
  
  private Euro summe = new Euro();
  
  public void addiereWert(Euro wert, boolean unterkat) {
    if(unterkat)
      summe.sum(wert);
    else
      getHauptkategorie().summe.sum(wert);
  }
  
  public Euro getSumme() {
    return summe;
  }
  
  public void loescheSumme() {
    summe = new Euro();
  }
  
  // -- Methoden fuer Interface: Comparable -----------------------------------

  public int compareTo(Object obj) {
    Kategorie kategorie = (Kategorie) obj;
    int compare = getHauptkategorie().getName().compareToIgnoreCase(kategorie.getHauptkategorie().getName());
    if(compare != 0)
      return compare;
    // else: gleiche Hauptkategorien
    return toString().compareToIgnoreCase(""+obj);
  }

 }