/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2004 Dr. Lars H. Hahn

*/

package haushalt.daten;

import haushalt.daten.zeitraum.AbstractZeitraum;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * Eine Umbuchung verschiebt Geld von einem Register in ein anderes. Es entstehen
 * keine Einnahmen oder Ausgaben. Umbuchungen sind deshalb keiner Kategorie
 * zugeordnet. Statt der Kategorie besitzen Umbuchungen ein Register-Paar
 * (Quell- und Zielregister). Umbuchungen werden (bei nderungen) automatisch in 
 * das Quell- und das Zielregister einsortiert.
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */
public class Umbuchung extends AbstractBuchung {
	private final boolean DEBUG = false;

  private Euro wert = new Euro();
  private Register[] register = new Register[2]; // Quell- und Zielregister

  public Umbuchung() {
    register[0] = null;
    register[1] = null;
  }

  public Umbuchung(Datum datum, String text, Register[] register, Euro wert) {
    setDatum(datum);
    setText(text);
    setWert(wert);
    setKategorie(register);
  }

  // -- Kategorie --------------------------------------------------------------

  public Object getKategorie() {
    return register;
  }

  public void setKategorie(Object kategorie) {
    Register[] neueRegister = (Register[])kategorie;
    // Wenn sich das Partner-Register ndert, muss die Buchung entfernt und
    // wieder eingefgt werden. Vorsicht bei Selbstbuchungen!
    if(DEBUG)
    	System.out.println("Umbuchung.setKategorie: NEU "+neueRegister[0]+"/"+neueRegister[1]+"; ALT "+register[0]+"/"+register[1]);
    for(int i=0;i<2;i++) {
      if(((register[0] != register[1]) || (i == 0)) && (register[i] != null))
        register[i].loescheUmbuchung(this);
        // Wenn die alte Umbuchung eine Selbstbuchung war, darf nur 1x gelscht
        // werden.
      if((neueRegister[0] != neueRegister[1]) || (i == 0))
        neueRegister[i].einsortierenBuchung(this);
				// Wenn eine Buchung eine Selbstbuchung wird, muss sie einmal nur
				// einsortiert werden.
    }
    register = neueRegister;
  }
  
  public int ersetzeKategorie(Kategorie alteKategorie, Kategorie neueKategorie) {
    // Umbuchungen haben keine Kategorie!
    return 0;
  }

  // -- Buchungswert -----------------------------------------------------------

  public void setWert(Euro wert) {
    this.wert = wert;
  }
  
  public Euro getWert() {
    return wert;
  }
  
  // -- Auswertung -------------------------------------------------------------

  public void bildeKategorieSumme(AbstractZeitraum zeitraum, boolean unterkat) {
		// Umbuchungen haben keine Kategorie!
  }
  
	public Euro getKategorieWert(Kategorie namekat, boolean unterkat) {
		// Umbuchungen haben keine Kategorie!
    return Euro.NULL_EURO;
	}

  // -- E/A-Funktionen ---------------------------------------------------------

  public void laden(DataInputStream in, Datenbasis db)
    throws IOException {
		getDatum().laden(in);
		setText(in.readUTF());
    register[0] = db.findeOderErzeugeRegister(in.readUTF());
    wert.laden(in);
		if(DEBUG)
			System.out.println("Umbuchung: "+getText()+" / "+register[0]+" geladen.");
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    out.writeUTF("Umbuchung");
		getDatum().speichern(out);
		out.writeUTF(getText());
    out.writeUTF(""+register[0]);
    wert.speichern(out);
		if(DEBUG)
			System.out.println("Umbuchung: "+getText()+" / "+register[0]+" gespeichert.");
  }

  // -- Methode des Interface 'Cloneable' --------------------------------------

  public Object clone() {
    return new Umbuchung((Datum)getDatum().clone(), ""+getText(), (Register[])register.clone(), (Euro)getWert().clone());
  }

}